"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonApiJs = JsonApiJs;
const tslib_1 = require("tslib");
const utils_1 = require("./utils");
const service_1 = require("./service");
const fetch_inner_client_1 = require("./service/fetch-inner-client");
const rxjs_1 = require("rxjs");
const jsonSdk = {};
function JsonApiJs(config, returnPromise) {
    if (Object.keys(jsonSdk).length > 0)
        return jsonSdk;
    const { adapter } = config, jsonSdkConfig = tslib_1.__rest(config, ["adapter"]);
    const resultJsonConfig = (0, utils_1.resultConfig)(jsonSdkConfig);
    const jsonApiUtilsService = new service_1.JsonApiUtilsService(resultJsonConfig);
    const httpInnerClient = adapter ? adapter : new fetch_inner_client_1.FetchInnerClient();
    const jonApiSdkService = new service_1.JsonApiSdkService(httpInnerClient, jsonApiUtilsService, resultJsonConfig);
    jsonSdk['jsonApiUtilsService'] = jsonApiUtilsService;
    jsonSdk['jonApiSdkService'] = jonApiSdkService;
    jsonSdk['atomicFactory'] = () => new service_1.AtomicOperationsService(jsonApiUtilsService, resultJsonConfig, httpInnerClient);
    if (returnPromise) {
        jsonSdk['jonApiSdkService'] = new Proxy(jonApiSdkService, {
            get(target, p) {
                return (...arg) => (0, rxjs_1.lastValueFrom)(target[p].apply(target, arg));
            },
        });
        jsonSdk['atomicFactory'] = () => {
            const proxy = new Proxy(new service_1.AtomicOperationsService(jsonApiUtilsService, resultJsonConfig, httpInnerClient), {
                get(target, p) {
                    if (p === 'run') {
                        return () => (0, rxjs_1.lastValueFrom)(target.run());
                    }
                    if (typeof target[p] === 'function') {
                        return (...arg) => {
                            // @ts-ignore
                            target[p](...arg);
                            return proxy;
                        };
                    }
                    return target[p];
                },
            });
            return proxy;
        };
    }
    return jsonSdk;
}
//# sourceMappingURL=json-api-js.js.map