import { RelationKeys, EntityClass } from '@yaser2us/json-api-nestjs-shared';
import { Observable } from 'rxjs';
import { HttpInnerClient, JsonApiSdkConfig, QueryParams, QueryParamsForOneItem, ReturnIfArray } from '../types';
import { EntityArray } from '../utils';
import { JsonApiUtilsService } from './json-api-utils.service';
export declare class JsonApiSdkService {
    private http;
    private jsonApiUtilsService;
    private jsonApiSdkConfig;
    constructor(http: HttpInnerClient, jsonApiUtilsService: JsonApiUtilsService, jsonApiSdkConfig: JsonApiSdkConfig);
    getList<Entity extends object, IdKey extends string = 'id'>(entity: EntityClass<Entity>, params?: QueryParams<Entity>): Observable<EntityArray<Entity>>;
    getAll<Entity extends object>(entity: EntityClass<Entity>, params?: QueryParams<Entity>, push?: boolean): Observable<EntityArray<Entity>>;
    getOne<Entity extends object>(entity: EntityClass<Entity>, id: string | number, params?: QueryParamsForOneItem<Entity>): Observable<Entity>;
    postOne<Entity extends object>(entity: Entity): Observable<Entity>;
    patchOne<Entity extends object>(entity: Entity): Observable<Entity>;
    deleteOne<Entity extends object>(entity: Entity): Observable<void>;
    getRelationships<Entity extends object, IdKey extends string = 'id', Rel extends RelationKeys<Entity, IdKey> = RelationKeys<Entity, IdKey>>(entity: Entity, relationType: Rel): Observable<ReturnIfArray<Entity[Rel], string>>;
    patchRelationships<Entity extends object, IdKey extends string, Rel extends RelationKeys<Entity, IdKey>>(entity: Entity, relationType: Rel): Observable<ReturnIfArray<Entity[Rel], string>>;
    postRelationships<Entity extends object, IdKey extends string, Rel extends RelationKeys<Entity, IdKey>>(entity: Entity, relationType: Rel): Observable<ReturnIfArray<Entity[Rel], string>>;
    deleteRelationships<Entity extends object, IdKey extends string, Rel extends RelationKeys<Entity, IdKey>>(entity: Entity, relationType: Rel): Observable<void>;
}
