"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenerateAtomicBody = void 0;
const json_api_nestjs_shared_1 = require("@yaser2us/json-api-nestjs-shared");
const utils_1 = require("./utils");
class GenerateAtomicBody {
    constructor(jsonApiUtilsService, jsonApiSdkConfig) {
        this.jsonApiUtilsService = jsonApiUtilsService;
        this.jsonApiSdkConfig = jsonApiSdkConfig;
        this.skipEmpty = true;
    }
    get isSkipEmpty() {
        return this.skipEmpty;
    }
    setToBody(op, entity, relationType) {
        const type = (0, utils_1.getTypeForReq)(entity.constructor.name);
        const { relationships, attributes } = this.jsonApiUtilsService.generateBody(entity);
        const id = Reflect.get(entity, this.jsonApiSdkConfig.idKey);
        const idObj = op === 'add' && !relationType ? {} : { id: String(id) };
        const data = relationType
            ? this.jsonApiUtilsService.generateRelationshipsBody(entity[relationType])
            : Object.assign(Object.assign({ type: (0, utils_1.getTypeForReq)(entity.constructor.name), attributes }, (Object.keys(relationships).length > 0 ? { relationships } : {})), idObj);
        const rel = relationType ? { relationship: String(relationType) } : {};
        const tmpId = op === 'add' && id && !relationType ? { tmpId: String(id) } : {};
        this.bodyData = Object.assign({ op, ref: Object.assign(Object.assign(Object.assign({ type }, idObj), rel), tmpId) }, (op === 'remove' && !relationType ? {} : { data }));
    }
    getBody() {
        return this.bodyData;
    }
    postOne(entity) {
        this.setToBody(json_api_nestjs_shared_1.Operation.add, entity);
    }
    patchOne(entity) {
        if (Reflect.get(entity, this.jsonApiSdkConfig.idKey) === undefined) {
            throw new Error('Resource params should be instance of resource with id params');
        }
        this.setToBody(json_api_nestjs_shared_1.Operation.update, entity);
    }
    deleteOne(entity, skipEmpty) {
        if (!Reflect.get(entity, this.jsonApiSdkConfig.idKey)) {
            throw new Error('Resource params should be instance of resource with id params');
        }
        this.skipEmpty = skipEmpty;
        this.setToBody(json_api_nestjs_shared_1.Operation.remove, entity);
    }
    patchRelationships(entity, relationType) {
        if (!Reflect.get(entity, this.jsonApiSdkConfig.idKey)) {
            throw new Error('Resource params should be instance of resource with id params');
        }
        if (entity[relationType] === undefined) {
            new Error(`${relationType.toString()} should not be undefined in entity`);
        }
        this.setToBody(json_api_nestjs_shared_1.Operation.update, entity, relationType);
    }
    postRelationships(entity, relationType) {
        if (!Reflect.get(entity, this.jsonApiSdkConfig.idKey)) {
            throw new Error('Resource params should be instance of resource with id params');
        }
        if (entity[relationType] === undefined) {
            throw new Error(`${relationType.toString()} should not be undefined in entity`);
        }
        this.setToBody(json_api_nestjs_shared_1.Operation.add, entity, relationType);
    }
    deleteRelationships(entity, relationType) {
        if (!Reflect.get(entity, this.jsonApiSdkConfig.idKey)) {
            throw new Error('Resource params should be instance of resource with id params');
        }
        if (entity[relationType] === undefined) {
            throw new Error(`${relationType.toString()} should not be undefined in entity`);
        }
        this.setToBody(json_api_nestjs_shared_1.Operation.remove, entity, relationType);
    }
}
exports.GenerateAtomicBody = GenerateAtomicBody;
//# sourceMappingURL=generate-atomic-body.js.map