import { resultConfig } from './utils';
import { AtomicOperationsService, JsonApiSdkService, JsonApiUtilsService, } from './service';
import { FetchInnerClient } from './service/fetch-inner-client';
import { lastValueFrom } from 'rxjs';
const jsonSdk = {};
export function JsonApiJs(config, returnPromise) {
    if (Object.keys(jsonSdk).length > 0)
        return jsonSdk;
    const { adapter, ...jsonSdkConfig } = config;
    const resultJsonConfig = resultConfig(jsonSdkConfig);
    const jsonApiUtilsService = new JsonApiUtilsService(resultJsonConfig);
    const httpInnerClient = adapter ? adapter : new FetchInnerClient();
    const jonApiSdkService = new JsonApiSdkService(httpInnerClient, jsonApiUtilsService, resultJsonConfig);
    jsonSdk['jsonApiUtilsService'] = jsonApiUtilsService;
    jsonSdk['jonApiSdkService'] = jonApiSdkService;
    jsonSdk['atomicFactory'] = () => new AtomicOperationsService(jsonApiUtilsService, resultJsonConfig, httpInnerClient);
    if (returnPromise) {
        jsonSdk['jonApiSdkService'] = new Proxy(jonApiSdkService, {
            get(target, p) {
                return (...arg) => lastValueFrom(target[p].apply(target, arg));
            },
        });
        jsonSdk['atomicFactory'] = () => {
            const proxy = new Proxy(new AtomicOperationsService(jsonApiUtilsService, resultJsonConfig, httpInnerClient), {
                get(target, p) {
                    if (p === 'run') {
                        return () => lastValueFrom(target.run());
                    }
                    if (typeof target[p] === 'function') {
                        return (...arg) => {
                            // @ts-ignore
                            target[p](...arg);
                            return proxy;
                        };
                    }
                    return target[p];
                },
            });
            return proxy;
        };
    }
    return jsonSdk;
}
//# sourceMappingURL=json-api-js.js.map