import { KEY_MAIN_INPUT_SCHEMA, KEY_MAIN_OUTPUT_SCHEMA, } from '@yaser2us/json-api-nestjs-shared';
import { map } from 'rxjs/operators';
import { GenerateAtomicBody } from '../utils';
export class AtomicOperationsService {
    jsonApiUtilsService;
    jsonApiSdkConfig;
    httpInnerClient;
    generateAtomicBody = [];
    constructor(jsonApiUtilsService, jsonApiSdkConfig, httpInnerClient) {
        this.jsonApiUtilsService = jsonApiUtilsService;
        this.jsonApiSdkConfig = jsonApiSdkConfig;
        this.httpInnerClient = httpInnerClient;
    }
    addBody(item) {
        this.generateAtomicBody.push(item);
    }
    run() {
        if (!this.jsonApiSdkConfig.operationUrl) {
            throw new Error('Should be set operation url');
        }
        const atomicBody = this.generateAtomicBody.map((i) => i.getBody());
        const body = {
            [KEY_MAIN_INPUT_SCHEMA]: atomicBody,
        };
        const bodyWithoutDelete = atomicBody.filter((i) => i.op !== 'remove');
        const operationUrl = this.jsonApiUtilsService.getUrlForResource(this.jsonApiSdkConfig.operationUrl);
        const indexDeleteIfNotSkipEmpty = atomicBody
            .reduce((acum, item, index) => {
            if (item.op === 'remove' &&
                !this.generateAtomicBody[index].isSkipEmpty) {
                acum.push(index);
            }
            return acum;
        }, [])
            .sort((a, b) => a - b);
        return this.httpInnerClient.post(operationUrl, body).pipe(map((r) => r[KEY_MAIN_OUTPUT_SCHEMA]), map((r) => r.map((item, index) => bodyWithoutDelete[index].ref.relationship
            ? this.jsonApiUtilsService.getResultForRelation(item)
            : this.jsonApiUtilsService.convertResponseData(item))), map((r) => indexDeleteIfNotSkipEmpty.reduce((acc, index, currentIndex) => {
            acc.splice(index + currentIndex, 0, 'EMPTY');
            return acc;
        }, [...r])));
    }
    deleteOne(entity, skipEmpty) {
        return this.setToBody('deleteOne', entity, skipEmpty === undefined ? true : skipEmpty);
    }
    patchOne(entity) {
        return this.setToBody('patchOne', entity);
    }
    postOne(entity) {
        return this.setToBody('postOne', entity);
    }
    deleteRelationships(entity, relationType) {
        return this.setToBody('deleteRelationships', entity, relationType);
    }
    patchRelationships(entity, relationType) {
        return this.setToBody('patchRelationships', entity, relationType);
    }
    postRelationships(entity, relationType) {
        return this.setToBody('postRelationships', entity, relationType);
    }
    setToBody(operationType, entity, relationType) {
        const atomicBody = new GenerateAtomicBody(this.jsonApiUtilsService, this.jsonApiSdkConfig);
        if (typeof relationType === 'boolean') {
            if (operationType === 'deleteOne') {
                atomicBody[operationType](entity, relationType);
            }
        }
        else {
            switch (operationType) {
                case 'postRelationships':
                case 'patchRelationships':
                case 'deleteRelationships':
                    if (relationType)
                        atomicBody[operationType](entity, relationType);
                    break;
                default:
                    atomicBody[operationType](entity, true);
                    break;
            }
        }
        this.addBody(atomicBody);
        return this;
    }
}
//# sourceMappingURL=atomic-operations.service.js.map