import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
class AxiosHttpClient {
    axios;
    constructor(axios) {
        this.axios = axios;
    }
    observify(makeRequest, controller) {
        return new Observable((subscriber) => {
            makeRequest()
                .then((response) => subscriber.next(response))
                .catch((error) => subscriber.error(error))
                .finally(() => subscriber.complete());
            return { unsubscribe: () => controller.abort() };
        });
    }
    makeCancellable() {
        const controller = new AbortController();
        const signal = controller.signal;
        return {
            controller,
            signal,
        };
    }
    delete(url, body) {
        if (body) {
            return this.request(url, 'DELETE', body);
        }
        return this.request(url, 'DELETE');
    }
    get(url, params) {
        return this.request(url, 'GET', undefined, params ? params.params : undefined);
    }
    patch(url, body) {
        return this.request(url, 'PATCH', body);
    }
    post(url, body) {
        return this.request(url, 'POST', body);
    }
    request(url, method, data, params) {
        const { controller, signal } = this.makeCancellable();
        return this.observify(() => this.axios.request({
            url,
            method,
            data,
            signal,
            params,
        }), controller).pipe(map((r) => r.data));
    }
}
export function adapterForAxios(axios) {
    return new AxiosHttpClient(axios);
}
//# sourceMappingURL=adapter-for-axios.js.map