import { Operation } from '@yaser2us/json-api-nestjs-shared';
import { getTypeForReq } from './utils';
export class GenerateAtomicBody {
    jsonApiUtilsService;
    jsonApiSdkConfig;
    constructor(jsonApiUtilsService, jsonApiSdkConfig) {
        this.jsonApiUtilsService = jsonApiUtilsService;
        this.jsonApiSdkConfig = jsonApiSdkConfig;
    }
    bodyData;
    skipEmpty = true;
    get isSkipEmpty() {
        return this.skipEmpty;
    }
    setToBody(op, entity, relationType) {
        const type = getTypeForReq(entity.constructor.name);
        const { relationships, attributes } = this.jsonApiUtilsService.generateBody(entity);
        const id = Reflect.get(entity, this.jsonApiSdkConfig.idKey);
        const idObj = op === 'add' && !relationType ? {} : { id: String(id) };
        const data = relationType
            ? this.jsonApiUtilsService.generateRelationshipsBody(entity[relationType])
            : {
                type: getTypeForReq(entity.constructor.name),
                attributes,
                ...(Object.keys(relationships).length > 0 ? { relationships } : {}),
                ...idObj,
            };
        const rel = relationType ? { relationship: String(relationType) } : {};
        const tmpId = op === 'add' && id && !relationType ? { tmpId: String(id) } : {};
        this.bodyData = {
            op,
            ref: { type, ...idObj, ...rel, ...tmpId },
            ...(op === 'remove' && !relationType ? {} : { data }),
        };
    }
    getBody() {
        return this.bodyData;
    }
    postOne(entity) {
        this.setToBody(Operation.add, entity);
    }
    patchOne(entity) {
        if (Reflect.get(entity, this.jsonApiSdkConfig.idKey) === undefined) {
            throw new Error('Resource params should be instance of resource with id params');
        }
        this.setToBody(Operation.update, entity);
    }
    deleteOne(entity, skipEmpty) {
        if (!Reflect.get(entity, this.jsonApiSdkConfig.idKey)) {
            throw new Error('Resource params should be instance of resource with id params');
        }
        this.skipEmpty = skipEmpty;
        this.setToBody(Operation.remove, entity);
    }
    patchRelationships(entity, relationType) {
        if (!Reflect.get(entity, this.jsonApiSdkConfig.idKey)) {
            throw new Error('Resource params should be instance of resource with id params');
        }
        if (entity[relationType] === undefined) {
            new Error(`${relationType.toString()} should not be undefined in entity`);
        }
        this.setToBody(Operation.update, entity, relationType);
    }
    postRelationships(entity, relationType) {
        if (!Reflect.get(entity, this.jsonApiSdkConfig.idKey)) {
            throw new Error('Resource params should be instance of resource with id params');
        }
        if (entity[relationType] === undefined) {
            throw new Error(`${relationType.toString()} should not be undefined in entity`);
        }
        this.setToBody(Operation.add, entity, relationType);
    }
    deleteRelationships(entity, relationType) {
        if (!Reflect.get(entity, this.jsonApiSdkConfig.idKey)) {
            throw new Error('Resource params should be instance of resource with id params');
        }
        if (entity[relationType] === undefined) {
            throw new Error(`${relationType.toString()} should not be undefined in entity`);
        }
        this.setToBody(Operation.remove, entity, relationType);
    }
}
//# sourceMappingURL=generate-atomic-body.js.map